@@ -29,6 +29,13 @@ gboolean gst_gl_context_gen_shader (GstGLContext * context,
     const gchar * shader_vertex_source,
     const gchar * shader_fragment_source, GstGLShader ** shader);
 
+/* This is public API in 1.28 */
+#define gst_meta_api_type_tags_contain_only(api,valid_tags) \
+  gst_gl_gst_meta_api_type_tags_contain_only(api,valid_tags)
+
+gboolean gst_gl_gst_meta_api_type_tags_contain_only (GType api,
+    const gchar ** valid_tags);
+
 G_END_DECLS
 
 #endif /* __EXT_GL_GST_GL_UTILS_H__ */
