@@ -647,8 +647,7 @@ GST_START_TEST (test_buffer_clip_samples_start_and_stop_no_meta)
   fail_unless_equals_int64 (GST_BUFFER_TIMESTAMP (ret), 4 * GST_SECOND);
   fail_unless_equals_int64 (GST_BUFFER_DURATION (ret), GST_CLOCK_TIME_NONE);
   fail_unless_equals_int64 (GST_BUFFER_OFFSET (ret), 400);
-  fail_unless_equals_int64 (GST_BUFFER_OFFSET_END (ret),
-      GST_BUFFER_OFFSET_NONE);
+  fail_unless_equals_int64 (GST_BUFFER_OFFSET_END (ret), 800);
   gst_buffer_map (ret, &map, GST_MAP_READ);
   fail_unless (map.data == data + 200);
   fail_unless (map.size == 400);
@@ -664,7 +663,7 @@ GST_START_TEST (test_buffer_clip_samples_no_timestamp)
   GstSegment s;
   GstBuffer *buf;
 
-  /* If the buffer has no offset it should assert()
+  /* If the buffer has no offset it should assert() in DEFAULT format
    * FIXME: check if return value is the same as the input buffer.
    *        probably can't be done because the assert() does a SIGABRT.
    */
@@ -683,6 +682,57 @@ GST_START_TEST (test_buffer_clip_samples_no_timestamp)
 
 GST_END_TEST;
 
+GST_START_TEST (test_buffer_truncate_samples_offset_end)
+{
+  GstBuffer *buf;
+  GstBuffer *ret;
+  guint8 *data;
+
+  buf = make_buffer (&data);
+  GST_BUFFER_TIMESTAMP (buf) = 2 * GST_SECOND;
+  GST_BUFFER_DURATION (buf) = 10 * GST_SECOND;
+  GST_BUFFER_OFFSET (buf) = 200;
+  GST_BUFFER_OFFSET_END (buf) = 1200;
+
+  ret = gst_audio_buffer_truncate (buf, 4, 100, 100);
+  fail_unless (ret != NULL);
+
+  fail_unless_equals_int64 (GST_BUFFER_TIMESTAMP (ret), GST_CLOCK_TIME_NONE);
+  fail_unless_equals_int64 (GST_BUFFER_DURATION (ret), GST_CLOCK_TIME_NONE);
+  fail_unless_equals_int64 (GST_BUFFER_OFFSET (ret), 300);
+  fail_unless_equals_int64 (GST_BUFFER_OFFSET_END (ret), 400);
+
+  gst_buffer_unref (ret);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_buffer_truncate_samples_no_timestamp)
+{
+  GstBuffer *buf;
+  GstBuffer *ret;
+  guint8 *data;
+
+  buf = make_buffer (&data);
+  GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
+  GST_BUFFER_DURATION (buf) = GST_CLOCK_TIME_NONE;
+  GST_BUFFER_OFFSET (buf) = GST_BUFFER_OFFSET_NONE;
+  GST_BUFFER_OFFSET_END (buf) = GST_BUFFER_OFFSET_NONE;
+
+  ret = gst_audio_buffer_truncate (buf, 4, 100, 1);
+  fail_unless (ret != NULL);
+
+  fail_unless_equals_int64 (GST_BUFFER_TIMESTAMP (ret), GST_CLOCK_TIME_NONE);
+  fail_unless_equals_int64 (GST_BUFFER_DURATION (ret), GST_CLOCK_TIME_NONE);
+  fail_unless_equals_int64 (GST_BUFFER_OFFSET (ret), GST_BUFFER_OFFSET_NONE);
+  fail_unless_equals_int64 (GST_BUFFER_OFFSET_END (ret),
+      GST_BUFFER_OFFSET_NONE);
+
+  gst_buffer_unref (ret);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_multichannel_checks)
 {
   GstAudioChannelPosition pos_2_mixed[2] = {
@@ -926,7 +976,7 @@ GST_START_TEST (test_fill_silence)
   for (f = GST_AUDIO_FORMAT_S8; f < GST_AUDIO_FORMAT_F64; f++) {
     gst_audio_info_set_format (&info, f, 48000, 1, NULL);
 
-    gst_audio_format_fill_silence (info.finfo, test_silence,
+    gst_audio_format_info_fill_silence (info.finfo, test_silence,
         GST_AUDIO_INFO_BPF (&info) * 4);
 
     for (i = 0; i < 4; i++)
@@ -955,17 +1005,17 @@ GST_START_TEST (test_stream_align)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     if (i == 0)
       fail_unless (discont);
     else
       fail_unless (!discont);
+    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
   }
 
   /* Drift forwards by 1ms per 10ms buffer for the first 40 buffers.
    * - after 40 buffers we're above alignment threshold
-   * - after 40 + 100 buffers we're at discont wait
+   * - after 40 + 96 buffers we're at discont wait
    */
   for (i = 0; i < 500; i++) {
     timestamp = 10 * GST_MSECOND * i;
@@ -977,15 +1027,15 @@ GST_START_TEST (test_stream_align)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i < 140) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i < 136) {
       if (i == 0)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     } else {
-      if (i == 140)
+      if (i == 136)
         fail_unless (discont);
       else
         fail_unless (!discont);
@@ -997,7 +1047,7 @@ GST_START_TEST (test_stream_align)
 
   /* Drift backwards by 1ms per 10ms buffer for the first 40 buffers.
    * - after 40 buffers we're above alignment threshold
-   * - after 40 + 100 buffers we're at discont wait
+   * - after 40 + 104 buffers we're at discont wait
    */
   for (i = 0; i < 500; i++) {
     timestamp = 10 * GST_MSECOND * i;
@@ -1009,15 +1059,15 @@ GST_START_TEST (test_stream_align)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i < 140) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i < 144) {
       if (i == 0)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     } else {
-      if (i == 140)
+      if (i == 144)
         fail_unless (discont);
       else
         fail_unless (!discont);
@@ -1030,7 +1080,7 @@ GST_START_TEST (test_stream_align)
 
   /* Shift all buffers but the first by 40ms
    * - after 1 buffers we're above alignment threshold
-   * - after 101 buffers we're at discont wait
+   * - after 97 buffers we're at discont wait
    */
   for (i = 0; i < 500; i++) {
     timestamp = 10 * GST_MSECOND * i;
@@ -1042,15 +1092,15 @@ GST_START_TEST (test_stream_align)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i < 101) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i < 97) {
       if (i == 0)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     } else {
-      if (i == 101)
+      if (i == 97)
         fail_unless (discont);
       else
         fail_unless (!discont);
@@ -1074,15 +1124,15 @@ GST_START_TEST (test_stream_align)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     if (i == 0)
       fail_unless (discont);
     else
       fail_unless (!discont);
+    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
   }
 
-  /* Shift every buffer 100 by 2: discont at buffer 200
+  /* Shift every buffer after 100 by 2s: discont at buffer 100
    */
   for (i = 0; i < 500; i++) {
     timestamp = 10 * GST_MSECOND * i;
@@ -1094,21 +1144,21 @@ GST_START_TEST (test_stream_align)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i < 200) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i < 100) {
       if (i == 0)
         fail_unless (discont);
       else
         fail_unless (!discont);
-    } else {
-      fail_unless_equals_uint64 (out_timestamp,
-          10 * GST_MSECOND * i + 2 * GST_SECOND);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
       fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
-      if (i == 200)
+    } else {
+      if (i == 100)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp,
+          10 * GST_MSECOND * i + 2 * GST_SECOND);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     }
   }
 
@@ -1135,12 +1185,12 @@ GST_START_TEST (test_stream_align_reverse)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     if (i == 499)
       fail_unless (discont);
     else
       fail_unless (!discont);
+    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
   }
 
   /* Drift forwards by 1ms per 10ms buffer for the first 40 buffers.
@@ -1157,15 +1207,15 @@ GST_START_TEST (test_stream_align_reverse)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i >= 500 - 140) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i >= 500 - 144) {
       if (i == 499)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     } else {
-      if (i == 499 - 140)
+      if (i == 499 - 144)
         fail_unless (discont);
       else
         fail_unless (!discont);
@@ -1189,15 +1239,15 @@ GST_START_TEST (test_stream_align_reverse)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i >= 500 - 140) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i >= 500 - 136) {
       if (i == 499)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     } else {
-      if (i == 499 - 140)
+      if (i == 499 - 136)
         fail_unless (discont);
       else
         fail_unless (!discont);
@@ -1210,7 +1260,7 @@ GST_START_TEST (test_stream_align_reverse)
 
   /* Shift all buffers but the first by 40ms
    * - after 1 buffers we're above alignment threshold
-   * - after 101 buffers we're at discont wait
+   * - after 106 buffers we're at discont wait
    */
   for (i = 499; i >= 0; i--) {
     timestamp = 10 * GST_MSECOND * i;
@@ -1222,15 +1272,15 @@ GST_START_TEST (test_stream_align_reverse)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i >= 500 - 101) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i >= 500 - 105) {
       if (i == 499)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     } else {
-      if (i == 499 - 101)
+      if (i == 499 - 105)
         fail_unless (discont);
       else
         fail_unless (!discont);
@@ -1254,15 +1304,15 @@ GST_START_TEST (test_stream_align_reverse)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     if (i == 499)
       fail_unless (discont);
     else
       fail_unless (!discont);
+    fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
+    fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
   }
 
-  /* Shift buffer 100 by 2: discont at buffer 200
+  /* Shift every buffer after 100 by 2s: discont at buffer 100
    */
   for (i = 499; i >= 0; i--) {
     timestamp = 10 * GST_MSECOND * i;
@@ -1274,21 +1324,21 @@ GST_START_TEST (test_stream_align_reverse)
         gst_audio_stream_align_process (align, discont, timestamp, 10,
         &out_timestamp, &out_duration, NULL);
 
-    if (i >= 500 - 200) {
-      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
-      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
+    if (i >= 500 - 100) {
       if (i == 499)
         fail_unless (discont);
       else
         fail_unless (!discont);
-    } else {
-      fail_unless_equals_uint64 (out_timestamp,
-          10 * GST_MSECOND * i + 2 * GST_SECOND);
+      fail_unless_equals_uint64 (out_timestamp, 10 * GST_MSECOND * i);
       fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
-      if (i == 499 - 200)
+    } else {
+      if (i == 499 - 100)
         fail_unless (discont);
       else
         fail_unless (!discont);
+      fail_unless_equals_uint64 (out_timestamp,
+          10 * GST_MSECOND * i + 2 * GST_SECOND);
+      fail_unless_equals_uint64 (out_duration, 10 * GST_MSECOND);
     }
   }
 
@@ -1528,6 +1578,106 @@ GST_START_TEST (test_audio_make_raw_caps)
 
 GST_END_TEST;
 
+GST_START_TEST (test_audio_meta_serialize)
+{
+  GstBuffer *buf;
+  GstAudioInfo info;
+  GstAudioMeta *meta;
+
+  GstAudioChannelPosition position[] = { GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
+    GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT
+  };
+  gst_audio_info_init (&info);
+  gst_audio_info_set_format (&info, GST_AUDIO_FORMAT_S32, 44100, 2, position);
+  info.layout = GST_AUDIO_LAYOUT_NON_INTERLEAVED;
+
+  /* Add audio meta */
+  gsize samples = 1;
+  gsize buf_size = 4 * 2 * samples;
+  buf = gst_buffer_new_and_alloc (buf_size);
+  gsize offsets[] = { 0, buf_size / 2 };
+  meta = gst_buffer_add_audio_meta (buf, &info, samples, offsets);
+
+  /* Serialize */
+  GByteArray *data = g_byte_array_new ();
+  fail_unless (gst_meta_serialize_simple ((GstMeta *) meta, data));
+  gst_buffer_unref (buf);
+
+  /* Create a new buffer */
+  buf = gst_buffer_new_and_alloc (buf_size);
+  guint32 consumed;
+  meta = (GstAudioMeta *) gst_meta_deserialize (buf, data->data, data->len,
+      &consumed);
+  fail_unless (meta);
+  fail_unless (consumed == data->len);
+  g_byte_array_unref (data);
+
+  /* Check meta's content */
+  g_assert_cmpint (meta->info.finfo->format, ==, GST_AUDIO_FORMAT_S32);
+  g_assert_cmpint (meta->info.layout, ==, GST_AUDIO_LAYOUT_NON_INTERLEAVED);
+  g_assert_cmpint (meta->info.rate, ==, 44100);
+  g_assert_cmpint (meta->info.channels, ==, 2);
+  g_assert_cmpint (meta->info.position[0], ==,
+      GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT);
+  g_assert_cmpint (meta->info.position[1], ==,
+      GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT);
+  g_assert_cmpuint (meta->samples, ==, samples);
+  g_assert_cmpuint (meta->offsets[0], ==, 0);
+  g_assert_cmpuint (meta->offsets[1], ==, buf_size / 2);
+
+  gst_buffer_unref (buf);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_audio_meta_serialize_65_chans)
+{
+  GstBuffer *buf;
+  GstAudioInfo info;
+  GstAudioMeta *meta;
+
+  /* With more than 64 channels we cannot have positions */
+  gst_audio_info_init (&info);
+  gst_audio_info_set_format (&info, GST_AUDIO_FORMAT_S32, 44100, 65, NULL);
+  info.layout = GST_AUDIO_LAYOUT_NON_INTERLEAVED;
+
+  /* Add audio meta */
+  gsize samples = 1;
+  gsize buf_size = 4 * 65 * samples;
+  buf = gst_buffer_new_and_alloc (buf_size);
+  gsize offsets[65];
+  for (int i = 0; i < 65; i++)
+    offsets[i] = i * samples * 4;
+  meta = gst_buffer_add_audio_meta (buf, &info, samples, offsets);
+
+  /* Serialize */
+  GByteArray *data = g_byte_array_new ();
+  fail_unless (gst_meta_serialize_simple ((GstMeta *) meta, data));
+  gst_buffer_unref (buf);
+
+  /* Create a new buffer */
+  buf = gst_buffer_new_and_alloc (buf_size);
+  guint32 consumed;
+  meta = (GstAudioMeta *) gst_meta_deserialize (buf, data->data, data->len,
+      &consumed);
+  fail_unless (meta);
+  fail_unless (consumed == data->len);
+  g_byte_array_unref (data);
+
+  /* Check meta's content */
+  g_assert_cmpint (meta->info.finfo->format, ==, GST_AUDIO_FORMAT_S32);
+  g_assert_cmpint (meta->info.layout, ==, GST_AUDIO_LAYOUT_NON_INTERLEAVED);
+  g_assert_cmpint (meta->info.rate, ==, 44100);
+  g_assert_cmpint (meta->info.channels, ==, 65);
+  g_assert_cmpuint (meta->samples, ==, samples);
+  for (int i = 0; i < 65; i++)
+    g_assert_cmpuint (meta->offsets[i], ==, i * samples * 4);
+
+  gst_buffer_unref (buf);
+}
+
+GST_END_TEST;
+
 static Suite *
 audio_suite (void)
 {
@@ -1554,6 +1704,8 @@ audio_suite (void)
   tcase_add_test (tc_chain, test_buffer_clip_samples_outside);
   tcase_add_test (tc_chain, test_buffer_clip_samples_start_and_stop_no_meta);
   tcase_add_test (tc_chain, test_buffer_clip_samples_no_timestamp);
+  tcase_add_test (tc_chain, test_buffer_truncate_samples_offset_end);
+  tcase_add_test (tc_chain, test_buffer_truncate_samples_no_timestamp);
   tcase_add_test (tc_chain, test_multichannel_checks);
   tcase_add_test (tc_chain, test_multichannel_reorder);
   tcase_add_test (tc_chain, test_audio_format_s8);
@@ -1564,6 +1716,8 @@ audio_suite (void)
   tcase_add_test (tc_chain, test_audio_buffer_and_audio_meta);
   tcase_add_test (tc_chain, test_audio_info_from_caps);
   tcase_add_test (tc_chain, test_audio_make_raw_caps);
+  tcase_add_test (tc_chain, test_audio_meta_serialize);
+  tcase_add_test (tc_chain, test_audio_meta_serialize_65_chans);
 
   return s;
 }
